extern crate clap;
use clap::{App, Arg, ArgMatches};
use std::io::prelude::*;
use std::net::{TcpListener, TcpStream};
use std::process;
use std::sync::mpsc::channel;
use std::thread;

use ctrlc;
use echo_ip::ThreadPool;

fn main() {
    let (tx, rx) = channel();
    ctrlc::set_handler(move || tx.send(()).expect("Could not send signal on channel."))
        .expect("Error setting Ctrl+C handler");
    thread::spawn(move || {
        let matches = handle_args();
        let host = matches.value_of("server").unwrap_or("0.0.0.0");
        let port = matches.value_of("port").unwrap_or("8888");
        create_listener(host, port);
    });
    rx.recv().expect("Could not receive from channel.");
    println!("Exiting...");
    process::exit(0);
}

fn create_listener(host: &str, port: &str) {
    let bind_address = host.to_owned() + ":" + port;
    let listener = TcpListener::bind(&bind_address);
    let pool = ThreadPool::new(4);
    match listener {
        Ok(n) => {
            println!("-----------------------------------------------------------");
            println!("Server listening on {}", &port);
            println!("-----------------------------------------------------------");
            for stream in n.incoming() {
                let stream = stream.unwrap();
                pool.execute(|| {
                    handle_connection(stream);
                });
            }
        }
        Err(_e) => println!("Address already in use: {}", bind_address),
    }
}

fn handle_args() -> ArgMatches<'static> {
    return App::new("echo-ip")
        .version("1.0")
        .author("Yaser Hsueh <xueye404@icloud.com>")
        .about("https://www.simaek.com")
        .arg(
            Arg::with_name("port")
                .short("p")
                .long("port")
                .value_name("port")
                .help("listening port")
                .takes_value(true),
        )
        .arg(
            Arg::with_name("server")
                .short("s")
                .long("server")
                .value_name("host")
                .help("listening address")
                .takes_value(true),
        )
        .get_matches();
}

fn handle_connection(mut stream: TcpStream) {
    let mut buffer = [0; 1024];
    let remote_addr = stream.peer_addr().unwrap().ip();
    //println!("{}", String::from_utf8_lossy(&buffer[..]));
    stream.read(&mut buffer).unwrap();
    let get = b"GET / HTTP/1.1\r\n";
    let (status_line, contents) = if buffer.starts_with(get) {
        ("HTTP/1.1 200 OK", remote_addr.to_string())
    } else {
        ("HTTP/1.1 404 NOT FOUND", "URL ERROR".to_string())
    };
    println!("echo {}", remote_addr);
    let response = format!("{}\r\n\r\n{}\r\n", status_line, contents);
    stream.write(response.as_bytes()).unwrap();
    stream.flush().unwrap();
}
